import os
import json
import logging
import numpy
import scipy.ndimage as ndimage
import pydicom as dicom
from pydicom.uid import (
    ExplicitVRLittleEndian,
    ImplicitVRLittleEndian,
    ExplicitVRBigEndian,
)

UNCOMPRESSED_SYNTAXES = [
    ImplicitVRLittleEndian,
    ExplicitVRLittleEndian,
    ExplicitVRBigEndian,
]

Logger = logging.getLogger(__name__)
SUCCESS = 0
FAIL = 1

class RotateImage:
    inDirPath = ''
    outDirPath = ''
    configDirPath = ''
    configFileName = 'RotateAngleConfig.json'
    
    def IsCompressed(self, dataset):
        transfer_syntax = dataset.file_meta.TransferSyntaxUID
        return transfer_syntax not in UNCOMPRESSED_SYNTAXES

    #Constructor
    def __init__(self, inDir, outDir, configDir):
        self.inDirPath = inDir
        self.outDirPath = outDir
        self.configDirPath = configDir
        
    #Get dicom images from given folder
    def GetDicomFiles(self, dicomFolder):
        try:
            files_list = []
            for root, directory, files in os.walk(dicomFolder):
                for file in files:
                    files_list.append(os.path.join(root, file))
            return files_list
        except Exception as ex:
            raise ex

    #Rotate image with given angle
    def RotateImageByAngle(self, dicomFileList, rotateAngle):
        datasetList = []

        for dicomFile in dicomFileList:
            try:
                [path,filename] = os.path.split(dicomFile)
                Logger.info('Reading file: %s ', filename)

                dataset = dicom.dcmread(dicomFile)               
                if 'PixelData' in dataset:
                    if self.IsCompressed(dataset):
                        dataset.decompress()
                    new_data = ndimage.rotate(dataset.pixel_array, - int(rotateAngle), reshape=False)
                    dataset.PixelData = new_data.tobytes()
                    datasetList.append(dataset)
                else:
                    Logger.warn('%s does not have pixel data', filename)
            except dicom.errors.InvalidDicomError:
                Logger.error('%s is not a valid dicom file', filename)
                continue
            except Exception as ex:
                Logger.exception('Exception occured during processing file %s', filename)
                continue

        return datasetList

    #Get Rotation angle from config file
    def GetRotateAngle(self):
        rotateAngle = 90 # default
        if(self.configDirPath):
            configFile = os.path.join(self.configDirPath,self.configFileName)
            if os.path.exists(configFile):
                try:
                    #Read 
                    with open(configFile, 'r') as cfgFile:
                        data=cfgFile.read()
                    #Parse 
                    obj = json.loads(data)
                    rotateAngle = obj['angle']
                except Exception:
                    Logger.exception('Error reading configuration file %s. Using default rotation angle %s', configFile, rotateAngle)
            else:
                Logger.warn('Config file: %s does not exist. Using default rotation angle %s', configFile, rotateAngle)
        return rotateAngle

    #this is very simplistic ==> do better in your real app!
    def createUID(self, UID):
        tempuid = UID.rsplit('.', maxsplit=1)
        tempuid = "{}.{}".format(tempuid[0], int(numpy.random.uniform(0, 1e44)))
        tempuid = tempuid[:64] # dicom standard only allows for 64 characters here. Let's hope this is unique enough. Again, do better in production!
        return tempuid

    #Save dicom images
    def SaveRotatedImages(self, datasetList):
        # put all output images to one series
        seriesUID = self.createUID(datasetList[0].SeriesInstanceUID)
        seriesDescription = 'RotatedImages'
        
        for dataset in datasetList:
            try:
                #Modify Series UID
                dataset.SeriesInstanceUID = seriesUID
                dataset.SeriesDescription = seriesDescription
                #Modify Instance UID
                instanceUID = self.createUID(dataset.SOPInstanceUID)
                dataset.file_meta.MediaStorageSOPInstanceUID = instanceUID
                dataset.SOPInstanceUID = instanceUID
                
                # modify modality to Secondary capture
                dataset.Modality = 'SC'
                dataset.file_meta.MediaStorageSOPClassUID = '1.2.840.10008.5.1.4.1.1.7'
                dataset.SOPClassUID = '1.2.840.10008.5.1.4.1.1.7'
                dataset.add_new(0x00080064, 'CS', 'SYN')  # required attribute for SC image (ConversionType)

                outFileName = os.path.join(self.outDirPath, instanceUID)
                dataset.save_as(outFileName)

                Logger.info('Saved rotated image %s successfully', outFileName)
            except Exception as ex:
                Logger.exception('Exception occured on saving dicom image %s', outFileName)
                if os.path.exists(outFileName):
                    os.remove(outFileName)
                continue

    def main(self):
        try:

            # Read configurations
            rotateAngle = self.GetRotateAngle()
            Logger.info('Rotate Angle is %s', rotateAngle)

            if not os.path.isdir(self.inDirPath) or not os.path.exists(self.inDirPath):
                Logger.error('Source dicom images folder: %s is not valid', self.inDirPath)
                return FAIL
            
            #Get source dicom images list
            dicomFiles = self.GetDicomFiles(self.inDirPath)
            
            #Rotate dicom images
            RotatedDatasetList = self.RotateImageByAngle(dicomFiles, rotateAngle)

            #Write to output folder
            self.SaveRotatedImages(RotatedDatasetList)

            return SUCCESS

        except Exception as ex:
            Logger.exception("Exception info:")
            return FAIL
